@@ -4,13 +4,17 @@ Created on Sep 17, 2014
 """
 import codecs
 import csv
-import os
-import typing
-
-from bi_etl.timer import Timer
-from bi_etl.scheduler.task import ETLTask
-from bi_etl.components.etlcomponent import ETLComponent
 import logging
+import os
+from typing import *
+from typing import TextIO
+from pathlib import Path
+
+from itertools import islice
+
+from bi_etl.components.etlcomponent import ETLComponent
+from bi_etl.scheduler.task import ETLTask
+from bi_etl.timer import Timer
 
 __all__ = ['CSVReader']
 
@@ -147,9 +151,12 @@ class CSVReader(ETLComponent):
         large_field_support: boolean
             Enable support for csv columns bigger than 131,072 default limit.
     """
+
+    DELIMITER_SNIFF_LINES = 10
+
     def __init__(self,
-                 task: typing.Optional[ETLTask],
-                 filedata: typing.Union[typing.TextIO, str],
+                 task: Optional[ETLTask],
+                 filedata: Union[TextIO, str, Path],
                  encoding: str = None,
                  errors: str = 'strict',
                  logical_name: str = None,
@@ -160,9 +167,12 @@ class CSVReader(ETLComponent):
         
         self.__close_file = False
         
+        if isinstance(filedata, Path):
+            filedata = str(filedata)
+        
         # We have to check / open the file here to get the name for the logical name
         if isinstance(filedata, str):
-            self.log.info("Opening file {}".format(filedata))
+            self.log.debug(f"Opening file {filedata}")
 
             if encoding is None or encoding in {'utf8', 'utf-8'}:
                 with open(filedata, 'rb') as binary_file:
@@ -198,7 +208,7 @@ class CSVReader(ETLComponent):
         
         # Begin csv module params        
         self.dialect = "excel"
-        # self.delimiter = ','
+        self.delimiter = None
         self.doublequote = True
         self.escapechar = None
         self.quotechar = '"'
@@ -238,11 +248,23 @@ class CSVReader(ETLComponent):
         Build or get the csv.reader object.
         """
         if self.__reader is None:
-            if not hasattr(self, 'delimiter'):
-                delimiters = [',', '|', '\t']
-                dialect = csv.Sniffer().sniff('\n'.join(self.file.readlines(10)), delimiters=delimiters)
+            if self.delimiter is None:
+                delimiters = ''.join([',', '|', '\t'])
+                try:
+                    # Note: The sniff routine builds a histogram for delimiter frequency
+                    #       So we want to make sure to give it whole lines and not cut the
+                    #       last line off in the middle.
+                    sample_data = ''.join(islice(self.file, self.DELIMITER_SNIFF_LINES))
+                    dialect = csv.Sniffer().sniff(sample_data, delimiters=delimiters)
+                except csv.Error as e:
+                    try:
+                        name = self.file.name
+                    except AttributeError:
+                        name = self.file
+                    raise ValueError(f"{e} in file {name} delimiters={delimiters}")
                 if dialect.delimiter not in delimiters:
-                    msg = f'Invalid delimiter \'{dialect.delimiter}\' found in {self.file.name} csv file.'
+                    msg = f'Invalid delimiter \'{dialect.delimiter}\' found in {self.file.name} csv file. ' \
+                          f'Expected one of {delimiters}'
                     self.log.warning(msg)
                     raise ValueError(msg)
                 else:
@@ -338,18 +360,18 @@ class CSVReader(ETLComponent):
             assert header_row is not None, "Header row not found (or empty)"
             # Make sure to use setter here! It deals with duplicate names                
             self.column_names = header_row
-            if self.trace_data:
-                self.log.debug("Column names read: {}".format(self.column_names))
+
         except StopIteration:
-            pass
+            self.column_names = []
+
+        if self.trace_data:
+            self.log.debug(f"Column names read: {self.column_names}")
     
     def _raw_rows(self):
         len_column_names = len(self.column_names)
         try:
             self.seek_row(self.start_row)
-            this_iteration_header = self.generate_iteration_header(
-                columns_in_order=self.column_names,
-            )
+            this_iteration_header = self.full_iteration_header
             # noinspection PyTypeChecker
             for row in self.reader:
                 if len(row) != 0:     
@@ -378,10 +400,11 @@ class CSVReader(ETLComponent):
             else:
                 raise e
         
-    def close(self):
+    def close(self, error: bool = False):
         """
         Close the file
         """
-        if self.__close_file:
-            self.file.close()
-        super(CSVReader, self).close()
+        if not self.is_closed:
+            if self.__close_file:
+                self.file.close()
+            super(CSVReader, self).close(error=error)
