@@ -1,9 +1,9 @@
-import os
-import toml
-from bi_etl.bi_config_parser import BIConfigParser
+from pathlib import Path
 
-package_root = BIConfigParser.get_package_root()
-poetry_config = toml.load(os.path.join(package_root, 'pyproject.toml'))
+import toml
+
+package_root = Path(__file__).parent.parent.absolute()
+poetry_config = toml.load(str(package_root / 'pyproject.toml'))
 
 full_version = poetry_config['tool']['poetry']['version']
 
